SkillDumper = {}

SkillDumper.name = "SkillDumper"
SkillDumper.savedData = {}

function SkillDumper:Initialize()
    -- Do some init stuff
    SkillDumper.savedData = ZO_SavedVars:NewAccountWide('SkillDumperSavedVariables', 1, nil, systemDefault, nil, 'SkillDumper')

    -- wait 10 seconds or so before we begin dumping so we don't do it right when we load in
    zo_callLater(function()
        CHAT_ROUTER:AddSystemMessage("SkillDumper starting")

        for k in pairs(SkillDumper.savedData) do
            SkillDumper.savedData[k] = nil
        end

        if not SkillDumper.savedData then
            SkillDumper.savedData = {}
        end

        local sdm = SKILLS_DATA_MANAGER

        --* SKILL_TYPE_ARMOR
        --* SKILL_TYPE_AVA
        --* SKILL_TYPE_CHAMPION
        --* SKILL_TYPE_CLASS
        --* SKILL_TYPE_GUILD
        --* SKILL_TYPE_NONE
        --* SKILL_TYPE_RACIAL
        --* SKILL_TYPE_TRADESKILL
        --* SKILL_TYPE_WEAPON
        --* SKILL_TYPE_WORLD
        local skillTypes = { SKILL_TYPE_ARMOR, SKILL_TYPE_AVA, SKILL_TYPE_CHAMPION, SKILL_TYPE_CLASS, SKILL_TYPE_GUILD, SKILL_TYPE_NONE, SKILL_TYPE_RACIAL, SKILL_TYPE_TRADESKILL, SKILL_TYPE_WEAPON, SKILL_TYPE_WORLD }
        local morphs = { MORPH_SLOT_MORPH_1, MORPH_SLOT_MORPH_2 }

        for i, skillType in ipairs(skillTypes) do
            if not SkillDumper.savedData[skillType] then
                SkillDumper.savedData[skillType] = {}
            end
            for skillLineIndex = 1, GetNumSkillLines(skillType) do
                local sld = sdm:GetSkillLineDataByIndices(skillType, skillLineIndex)
                if sld then
                    SkillDumper.savedData[skillType][skillLineIndex] = { name = sld:GetFormattedName(), type = skillType, points = 0 }

                    if IsWerewolfSkillLineById(GetSkillLineId(skillType, skillLineIndex)) then
                        SkillDumper.savedData[skillType][skillLineIndex]["isWerewolf"] = true
                        SkillDumper.savedData[skillType][skillLineIndex]["isWerewolfParent"] = true
                    end

                    for skillIndex = 1, GetNumSkillAbilities(skillType, skillLineIndex) do
                        local sd = sld:GetSkillDataByIndex(skillIndex)
                        if sd then
                            local function GetNextAbilityMechanicFlagIter(abilityId)
                                return function(_, lastFlag)
                                    return GetNextAbilityMechanicFlag(abilityId, lastFlag)
                                end
                            end

                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex] = {
                                type = skillType,
                                line = skillLineIndex,
                                index = skillIndex,
                                autoGrant = sd:IsAutoGrant(),
                                passive = sd:IsPassive(),
                            }

                            local skillProgressionData = sd:GetCurrentProgressionData()
                            if skillProgressionData then
                                local mechanicFlag = 1
                                for flag in GetNextAbilityMechanicFlagIter(skillProgressionData:GetAbilityId()) do
                                    mechanicFlag = flag
                                    break
                                end
                                local mechanicName = GetString("SI_COMBATMECHANICFLAGS", mechanicFlag)
                                for rank = 1, 4, 1 do
                                    if rank > 1 and SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank-1].descriptionText == GetAbilityDescription(skillProgressionData:GetAbilityId(), rank) and SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank-1].cost == GetAbilityCost(skillProgressionData:GetAbilityId(), mechanicFlag, rank) then
                                        break
                                    end
                                    if not SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank] then
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank] = {}
                                    end
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].descriptionText = GetAbilityDescription(skillProgressionData:GetAbilityId(), rank)
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].cost = GetAbilityCost(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    local cost, chargeFrequencyMS = GetAbilityCostOverTime(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    if cost > 0 then
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].costOverTime = cost
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].chargeFrequencyMS = chargeFrequencyMS
                                    end
                                    local channeled, castTime, channelTime = GetAbilityCastInfo(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].channeled = channeled
                                    if channeled then
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].channelTime = channelTime
                                    else
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].castTime = castTime
                                    end
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].targetDescription = GetAbilityTargetDescription(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    local minRangeCM, maxRangeCM = GetAbilityRange(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].minRangeCM = minRangeCM
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].maxRangeCM = maxRangeCM
                                    local radiusCM = GetAbilityRadius(skillProgressionData:GetAbilityId(), mechanicFlag, rank)
                                    local angleDistanceCM = GetAbilityAngleDistance(skillProgressionData:GetAbilityId())
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].radiusCM = radiusCM
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].angleDistanceCM = angleDistanceCM
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].isAbilityDurationToggled = IsAbilityDurationToggled(skillProgressionData:GetAbilityId())
                                    if not IsAbilityDurationToggled(skillProgressionData:GetAbilityId()) then
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].durationMS = GetAbilityDuration(skillProgressionData:GetAbilityId(), mechanicFlag)
                                    end
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].cooldownMS = GetAbilityCooldown(skillProgressionData:GetAbilityId())
                                    local isTankRole, isHealerRole, isDamageRole = GetAbilityRoles(skillProgressionData:GetAbilityId())
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].isTankRole = isTankRole
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].isHealerRole = isHealerRole
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex][rank].isDamageRole = isDamageRole
                                end
                                SkillDumper.savedData[skillType][skillLineIndex][skillIndex].isUltimate = IsAbilityUltimate(skillProgressionData:GetAbilityId())
                                SkillDumper.savedData[skillType][skillLineIndex][skillIndex].mechanicName = mechanicName
                            end

                            local baseData = SkillDumper.savedData[skillType][skillLineIndex][skillIndex].passive and sd:GetRankData(1) or sd:GetMorphData(MORPH_SLOT_BASE)
                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex].name = baseData:GetFormattedName()
                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex].texture = baseData:GetIcon()

                            if not SkillDumper.savedData[skillType][skillLineIndex][skillIndex].passive then
                                if not SkillDumper.savedData[skillType][skillLineIndex][skillIndex].morphs then
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex].morphs = {}
                                end
                                for z, morph in ipairs(morphs) do
                                    if not SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph] then
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph] = {}
                                    end
                                    morphData = sd:GetMorphData(morph)
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph].name = morphData:GetFormattedName()
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph].texture = morphData:GetIcon()
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph].abilityId = morphData:GetAbilityId()

                                    -- Here the code repeats for the ability, later I want to refactor this to make it cleaner but for now I just want a working concept
                                    local mechanicFlag = 1
                                    for flag in GetNextAbilityMechanicFlagIter(morphData:GetAbilityId()) do
                                        mechanicFlag = flag
                                        break
                                    end
                                    local mechanicName = GetString("SI_COMBATMECHANICFLAGS", mechanicFlag)
                                    for rank = 1, 4, 1 do
                                        if rank > 1 and SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank-1].descriptionText == GetAbilityDescription(morphData:GetAbilityId(), rank) and SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank-1].cost == GetAbilityCost(morphData:GetAbilityId(), mechanicFlag, rank) then
                                            break
                                        end
                                        if not SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank] then
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank] = {}
                                        end
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].descriptionText = GetAbilityDescription(morphData:GetAbilityId(), rank)
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].cost = GetAbilityCost(morphData:GetAbilityId(), mechanicFlag, rank)
                                        local cost, chargeFrequencyMS = GetAbilityCostOverTime(morphData:GetAbilityId(), mechanicFlag, rank)
                                        if cost > 0 then
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].costOverTime = cost
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].chargeFrequencyMS = chargeFrequencyMS
                                        end
                                        local channeled, castTime, channelTime = GetAbilityCastInfo(morphData:GetAbilityId(), mechanicFlag, rank)
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].channeled = channeled
                                        if channeled then
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].channelTime = channelTime
                                        else
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].castTime = castTime
                                        end
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].targetDescription = GetAbilityTargetDescription(morphData:GetAbilityId(), mechanicFlag, rank)
                                        local minRangeCM, maxRangeCM = GetAbilityRange(morphData:GetAbilityId(), mechanicFlag, rank)
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].minRangeCM = minRangeCM
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].maxRangeCM = maxRangeCM
                                        local radiusCM = GetAbilityRadius(morphData:GetAbilityId(), mechanicFlag, rank)
                                        local angleDistanceCM = GetAbilityAngleDistance(morphData:GetAbilityId())
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].radiusCM = radiusCM
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].angleDistanceCM = angleDistanceCM
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].isAbilityDurationToggled = IsAbilityDurationToggled(morphData:GetAbilityId())
                                        if not IsAbilityDurationToggled(morphData:GetAbilityId()) then
                                            SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].durationMS = GetAbilityDuration(morphData:GetAbilityId(), mechanicFlag)
                                        end
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].cooldownMS = GetAbilityCooldown(morphData:GetAbilityId())
                                        local isTankRole, isHealerRole, isDamageRole = GetAbilityRoles(morphData:GetAbilityId())
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].isTankRole = isTankRole
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].isHealerRole = isHealerRole
                                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph][rank].isDamageRole = isDamageRole
                                    end
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph].isUltimate = IsAbilityUltimate(morphData:GetAbilityId())
                                    SkillDumper.savedData[skillType][skillLineIndex][skillIndex]["morphs"][morph].mechanicName = mechanicName
                                end
                            end

                        end

                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].morph = not SkillDumper.savedData[skillType][skillLineIndex][skillIndex].passive and 0 or nil
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].points = 0
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].auxListIndex = 0
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].maxRaMo = false
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].rank = 1
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].hb = {}
                        SkillDumper.savedData[skillType][skillLineIndex][skillIndex].purchased = false
                    end

                end
            end
        end

        CHAT_ROUTER:AddSystemMessage("SkillDumper finished")
    end, 10500)
end

function SkillDumper.OnAddOnLoaded(event, addonName)
    if addonName == SkillDumper.name then
        SkillDumper:Initialize()
    end
end

EVENT_MANAGER:RegisterForEvent(SkillDumper.name, EVENT_ADD_ON_LOADED, SkillDumper.OnAddOnLoaded)